from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Optional


@dataclass
class LookupResponse:
    first_name: str
    last_name: str
    middle_name: str
    date_of_birth: str
    photo: str
    gender: str
    phone_number: str
    confidence_value: str
    match: bool
    nin: Optional[str] = ""

    def to_dict(self):
        # Todo: normalize date of birth
        gender = {
            "m": "Male",
            "male": "Male",
            "f": "Female",
            "female": "Female",
        }
        return {
            "first_name": self.first_name,
            "last_name": self.last_name,
            "middle_name": self.middle_name,
            "date_of_birth": self.date_of_birth,
            "photo": self.photo,
            "gender": gender[self.gender.lower()],
            "phone_number": self.phone_number,
            "match": self.match,
            "confidence_value": self.confidence_value,
            "nin": self.nin,
        }


class BaseIdentityService(ABC):
    @abstractmethod
    def lookup_bvn(self, bvn) -> LookupResponse:
        pass

    @abstractmethod
    def lookup_nin(self, nin) -> LookupResponse:
        pass

    @abstractmethod
    def lookup_advanced_bvn(self, bvn) -> LookupResponse:
        pass



    @abstractmethod
    def lookup_bvn_with_image(self, bvn, image) -> LookupResponse:
        pass





from urllib.parse import urlparse

import redis
from django.conf import settings
from django.db import connections, OperationalError
from rest_framework.decorators import api_view
from rest_framework.response import Response


@api_view(['GET'])
def readiness_check(request):
    response = {'database': 'unknown', 'redis': 'unknown'}

    # Check database connection
    try:
        db_conn = connections['default']
        db_conn.cursor()
        response['database'] = 'ready'
    except OperationalError:
        response['database'] = 'not ready'

    # Check Redis connection
    try:
        redis_url = urlparse(settings.REDIS_URL)
        r = redis.StrictRedis(
            host=redis_url.hostname,
            port=redis_url.port,
            password=redis_url.password,
            decode_responses=True
        )
        r.ping()
        response['redis'] = 'ready'
    except redis.ConnectionError:
        response['redis'] = 'not ready'

    return Response(response)


@api_view(['GET'])
def health_check(request):
    response = {'status': True}
    return Response(response)



from enum import Enum


class CustomEnum(Enum):
    @classmethod
    def values(cls):
        return [c.value for c in cls]

    @classmethod
    def choices(cls):
        return [(c.value, c.value) for c in cls]


from drf_spectacular.types import OpenApiTypes
from drf_spectacular.utils import extend_schema, OpenApiParameter

header = extend_schema(
    parameters=[OpenApiParameter(
        "X-Api-Key", OpenApiTypes.STR,
        OpenApiParameter.HEADER, required=True)
    ])



from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response

DEFAULT_PAGE = 1


class CustomPagination(PageNumberPagination):
    # page_size = settings.REST_FRAMEWORK['PAGE_SIZE']
    page_size_query_param = 'page_size'

    def get_paginated_response(self, data):
        return Response({
            'links': {
                'next': self.get_next_link(),
                'previous': self.get_previous_link()
            },
            'total': self.page.paginator.count,
            'total_pages': self.page.paginator.num_pages,
            'current_page': int(self.request.GET.get('page', DEFAULT_PAGE)),
            'page_size': int(self.request.GET.get('page_size', self.page_size)),
            'results': data
        })


class StandardResultsSetPagination(PageNumberPagination):
    page_size = 20
    page_size_query_param = 'page_size'
    max_page_size = 50



from rest_framework import serializers


class EnumCharField(serializers.CharField):
    def __init__(self, enum_class, *args, **kwargs):
        self.enum_class = enum_class
        super().__init__(*args, **kwargs)

    def to_representation(self, value):
        if value is not None:
            return str(value)
        return None

    def to_internal_value(self, data):
        if data is not None:
            try:
                return getattr(self.enum_class, data)
            except AttributeError:
                raise serializers.ValidationError(f"Invalid enum value: {data}")
        return None


class GenericRequestResponse(serializers.Serializer):
    success = serializers.BooleanField(default=True)
    message = serializers.CharField(default="Success")


class EmptySerializer(serializers.Serializer):
    pass

class DigitOnlyFieldSerializer(serializers.CharField):
    def to_internal_value(self, data):
        if not str(data).isdigit():
            raise serializers.ValidationError("This field should contain digits only.")
        return data


import json
from datetime import datetime

from bson import ObjectId


# class JSONEncoder(json.JSONEncoder):
#     def default(self, obj):
#         if isinstance(obj, ObjectId):
#             return str(obj)  # Convert ObjectId to string
#         return super(JSONEncoder, self).default(obj)

class MongoEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, ObjectId):
            return str(obj)  # Convert ObjectId to string
        if isinstance(obj, datetime):
            return obj.isoformat()  # Convert datetime to ISO format string
        # Use the default method for other types
        return super().default(obj)



from datetime import datetime

from bson import json_util
from django.conf import settings
from elasticsearch import Elasticsearch

from core.celery import APP

from pykolofinance.audtilog.mongo_connector import get_database

database = get_database()
app_name = settings.APP_NAME.lower()
db_name = settings.MONGODB_LOGGER_DATABASE
collection_name = database[db_name]


@APP.task()
def send_logs_to_logger(data):
    inserted_id = collection_name.insert_one(data).inserted_id
    # elastic_search = Elasticsearch(hosts=settings.MONGODB_LOGGER_URL)
    # result = elastic_search.index(index=settings.APP_NAME.lower(), body=data)
    return {
        'entry': str(inserted_id),
        'app_name': app_name,
        'db_name': db_name,
        'timestamp': str(datetime.now()),
    }


@APP.task()
def send_log_to_auditlogger(data):
    pass



import requests
from django.conf import settings

from .helpers import clean_termii_phone


def get_auth_token(request=None):
    if request is None:
        return {}
    if isinstance(request, dict):
        auth_token = request['headers']["Authorization"]
    else:
        auth_token = request.headers.get("Authorization")
    auth_token = auth_token.split()[1] if "Bearer" in auth_token else auth_token
    return {"Authorization": f"Bearer {auth_token}", "Content-Type": "application/json"}


def send_champ_sms(recipients, message):
    url = settings.SENDCHAMP_API_URL
    api_key = settings.SENDCHAMP_API_KEY
    sender = settings.SENDCHAMP_SENDER
    headers = {
        "accept": "application/json",
        "content-type": "application/json",
        "Authorization": f"Bearer {api_key}"
    }
    data = {
        'to': recipients,
        'message': message,
        'sender_name': sender,
        'route': 'dnd'
    }
    response = requests.post(url, headers=headers, data=data)
    return response.text


def send_termii_sms(recipient, message):
    url = f"{settings.TERMII_BASE_URL}/sms/send"
    payload = {
        "to": clean_termii_phone(recipient),
        "from": settings.TERMII_SENDER_ID,
        "sms": message,
        "type": "plain",
        "channel": "dnd",
        "api_key": settings.TERMII_API_KEY,
        "media": {
            "url": "https://media.example.com/file",
            "caption": "your media file"
        }
    }
    headers = {
        'Content-Type': 'application/json',
    }
    response = requests.request("POST", url, headers=headers, json=payload)
    return response.text


def send_termii_bulk_sms(recipients, message):
    url = f"{settings.TERMII_BASE_URL}/sms/send/bulk"
    payload = {
        "to": recipients,
        "from": settings.TERMII_SENDER_ID,
        "sms": message,
        "type": "plain",
        "channel": "generic",
        "api_key": settings.TERMII_API_KEY,
    }
    headers = {
        'Content-Type': 'application/json',
    }
    response = requests.request("POST", url, headers=headers, json=payload)
    return response.text



from datetime import datetime
from time import time
import random

from django.conf import settings

from cuid2 import Cuid

CUID_GENERATOR: Cuid = Cuid(length=24)


def generate_unique_id() -> str:
    return CUID_GENERATOR.generate()


def generate_reference_code(length=14, prefix=None) -> str:
    generator: Cuid = Cuid(length=length)
    code = generator.generate().upper()
    if prefix:
        return f"{prefix}{code}"
    return code




def __generate_transaction_ref():
    random_number = random.randint(1, 999)
    salt = str(random_number).zfill(3)
    time_ref = str(time()).replace('.', '')[6:15]

    return f'{salt}{time_ref}'


def generate_session_id():
    """
    30 chars long
    1-6 - Source Bank Code
    7-18 - date time in yymmddHHmmss, with HH in 24 hours format
    19-30 - 12 char unique code
    """
    bank_code = settings.KOLOMONI_INSTITUTION_CODE
    date_ref = datetime.now().strftime('%y%m%d%H%M%S')
    tx_ref = __generate_transaction_ref()
    x = f'{bank_code}{date_ref}{tx_ref}'
    print(len(x))
    return x



import uuid
from django.db import models

from .kgs import generate_unique_id


class AuditableModel(models.Model):
    id = models.CharField(max_length=50, primary_key=True, default=generate_unique_id, editable=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True



from .providers.base import BaseIdentityService
from .providers.dojah import DojahIdentityService
from .providers.mock import MockIdentityService


class IdentityFactory:
    def __init__(self, config=None):
        if config is None:
            config = {}
        self.config = config
        self.providers = {
            "DOJAH": DojahIdentityService,
            'MOCK': MockIdentityService
        }

    def get_service(self, provider) -> BaseIdentityService:
        return self.providers[provider](**self.config)



import base64

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import padding
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from django.conf import settings


class AthenaAESCrypter:
    KEY = settings.ATHENA_CLIENT_KEY
    IV = settings.ATHENA_CLIENT_SECRET

    def encrypt(self, plain_text):
        """
        Encrypts plain text using AES/CBC/PKCS5Padding.

        Args:
            plain_text (str): Text to encrypt.
            key (str): Encryption key (must be 16, 24, or 32 characters).
            iv (str): Initialization vector (must be 16 characters).

        Returns:
            str: Encrypted text (Base64-encoded).
        """

        # Convert key and IV to bytes
        key_bytes = self.KEY.encode('utf-8')[:16]
        iv_bytes = self.IV.encode('utf-8')[:16]
        plain_text_bytes = plain_text.encode('utf-8')

        # Create cipher object
        cipher = Cipher(algorithms.AES(key_bytes), modes.CBC(iv_bytes), backend=default_backend())
        encryptor = cipher.encryptor()

        # Apply PKCS7 padding
        paddler = padding.PKCS7(128).padder()
        padded_data = paddler.update(plain_text_bytes) + paddler.finalize()

        # Encrypt the data
        encrypted_data = encryptor.update(padded_data) + encryptor.finalize()

        # Return Base64-encoded encrypted text
        return base64.b64encode(encrypted_data).decode('utf-8')

    def decrypt(self, encrypted_text):
        """
        Decrypts encrypted text using AES/CBC/PKCS5Padding.

        Args:
            encrypted_text (str): Base64-encoded encrypted text.
            key (str): Encryption key (must be 16, 24, or 32 characters).
            iv (str): Initialization vector (must be 16 characters).

        Returns:
            str: Decrypted plain text.
        """

        # Convert key and IV to bytes
        key_bytes = self.KEY.encode('utf-8')[:16]
        iv_bytes = self.IV.encode('utf-8')[:16]
        encrypted_bytes = base64.b64decode(encrypted_text)

        # Create cipher object
        cipher = Cipher(algorithms.AES(key_bytes), modes.CBC(iv_bytes), backend=default_backend())
        decrypter = cipher.decryptor()

        # Decrypt and remove padding
        decrypted_padded_data = decrypter.update(encrypted_bytes) + decrypter.finalize()
        unpaddler = padding.PKCS7(128).unpadder()
        plain_text_bytes = unpaddler.update(decrypted_padded_data) + unpaddler.finalize()

        # Return the decrypted plain text
        return plain_text_bytes.decode('utf-8')



# -*- coding: utf-8 -*-

from django import forms
from django.contrib.postgres.fields import ArrayField
from django.contrib.postgres.forms import SplitArrayField
from django.core.exceptions import ValidationError
from django.db.models.fields.files import FieldFile, File
# from django.utils import six
from django.utils.translation import gettext_lazy as _


class MultiFileInput(forms.FileInput):

    def render(self, name, value, attrs={}):
        attrs['multiple'] = 'multiple'
        return super().render(name, None, attrs=attrs)

    def value_from_datadict(self, data, files, name):
        if hasattr(files, 'getlist'):
            return files.getlist(name)
        else:
            return [files.get(name)]


class MultiFileField(forms.FileField):
    widget = MultiFileInput
    default_error_messages = {
        'min_num': u"Ensure at least %(min_num)s files are uploaded (received %(num_files)s).",
        'max_num': u"Ensure at most %(max_num)s files are uploaded (received %(num_files)s).",
        'file_size': u"File: %(uploaded_file_name)s, exceeded maximum upload size."
    }

    def __init__(self, *args, **kwargs):
        self.min_num = kwargs.pop('min_num', 0)
        self.max_num = kwargs.pop('max_num', None)
        self.maximum_file_size = kwargs.pop('maximum_file_size', None)
        super().__init__(*args, **kwargs)

    def to_python(self, data):
        ret = []
        for item in data:
            ret.append(super().to_python(item))
        return ret

    def validate(self, data):
        super().validate(data)
        num_files = len(data)
        if len(data) and not data[0]:
            num_files = 0
        if num_files < self.min_num:
            raise ValidationError(self.error_messages['min_num'] % {'min_num': self.min_num, 'num_files': num_files})
            return
        elif self.max_num and num_files > self.max_num:
            raise ValidationError(self.error_messages['max_num'] % {'max_num': self.max_num, 'num_files': num_files})
        for uploaded_file in data:
            if self.maximum_file_size and uploaded_file.size > self.maximum_file_size:
                raise ValidationError(self.error_messages['file_size'] % {'uploaded_file_name': uploaded_file.name})


def to_file_object(field, instance, file):
    if isinstance(file, str) or file is None:
        return field.attr_class(instance, field, file)
    elif isinstance(file, File) and not isinstance(file, FieldFile):
        file_copy = field.attr_class(instance, field, file.name)
        file_copy.file = file
        file_copy._committed = False
        return file_copy
    elif isinstance(file, FieldFile) and not hasattr(file, 'field'):
        file.instance = instance
        file.field = field
        file.storage = field.storage
        return file
    else:
        return file


class ArrayFileDescriptor:

    def __init__(self, field):
        self.field = field

    def __get__(self, instance=None, owner=None):
        if instance is None:
            raise AttributeError(
                f"The '{self.field.name}' attribute can only be accessed from {owner.__name__} instances.")
        try:
            return [
                to_file_object(self.field.base_field, instance, file)
                for file in instance.__dict__[self.field.name]
            ]
        except Exception as ex:
            return []

    def __set__(self, instance, value):
        instance.__dict__[self.field.name] = value


class ArrayFileField(ArrayField):

    descriptor_class = ArrayFileDescriptor

    def set_attributes_from_name(self, name):
        super(ArrayField, self).set_attributes_from_name(name)
        self.base_field.set_attributes_from_name(f"{name}_array")

    def contribute_to_class(self, cls, name, **kwargs):
        super().contribute_to_class(cls, name, **kwargs)
        setattr(cls, self.name, self.descriptor_class(self))

    def pre_save(self, instance, add):
        """Returns field's value just before saving."""
        files = [
            to_file_object(self.base_field, instance, file)
            for file in super(ArrayField, self).pre_save(instance, add)
        ]

        for file_copy in files:
            if file_copy and not file_copy._committed:
                file_copy.save(file_copy.name, file_copy, save=False)

        return files

    def formfield(self, **kwargs):
        defaults = {
            'form_class': MultiFileField,
            'max_num': self.size
        }
        defaults.update(kwargs)
        return super(ArrayField, self).formfield(**defaults)



from django.conf import settings
from pymongo import MongoClient


def get_database():
    # Provide the mongodb atlas url to connect python to mongodb using pymongo
    # db_url = "mongodb://phlox:6GlbaXu0sJWUG@143.110.171.18:27017/PhloxLogDatabase?authSource=admin"
    db_url = settings.MONGODB_LOGGER_URL

    # Create a connection using MongoClient. You can import MongoClient or use pymongo.MongoClient
    client = MongoClient(db_url)
    database = settings.MONGODB_LOGGER_DATABASE
    # database = settings.APP_NAME.lower()

    return client[database]



import logging
from django.conf import settings
from rest_framework import permissions

logger = logging.getLogger(__name__)


class IsSuperAdmin(permissions.BasePermission):
    """Allows access only to admin users. """
    message = "Only Super Admins are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user and request.user.role == 'Super Admin')


class IsAdmin(permissions.BasePermission):
    """Allows access only to admin users. """
    message = "Only Admins are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user and request.user.role == 'Admin')


class IsAdminViewOnly(permissions.BasePermission):
    """Allows access only to admin users. """
    message = "Only Admins are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user and request.user.role == 'Admin View Only')


class IsAgentUser(permissions.BasePermission):
    """Allows access only to agents. """
    message = "Only Agent are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'Agent')

    def has_object_permission(self, request, view, obj):
        return self.has_permission(request, view)


class IsDivisionalHead(permissions.BasePermission):
    """Allows access only to agents. """
    message = "Only Divisional Head are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'Divisional Head')

    def has_object_permission(self, request, view, obj):
        return self.has_permission(request, view)


class IsAgentNetworkManager(permissions.BasePermission):
    """Allows access only to Agent Network Manager. """
    message = "Only Divisional Head are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'Agent Network Manager')


class IsRegionalHead(permissions.BasePermission):
    """Allows access only to Regional Head. """
    message = "Only Divisional Head are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'Regional Head')


class IsRelationshipOfficer(permissions.BasePermission):
    """Allows access only to Relationship Officer. """
    message = "Only Divisional Head are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'Relationship Officer')

    def has_object_permission(self, request, view, obj):
        return self.has_permission(request, view)


class IsMerchant(permissions.BasePermission):
    """Allows access only to Merchant. """
    message = "Only Divisional Head are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'Merchant')

    def has_object_permission(self, request, view, obj):
        return self.has_permission(request, view)


class IsSuperAgent(permissions.BasePermission):
    """Allows access only to Super Agent. """
    message = "Only Divisional Head are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'Super Agent')

    def has_object_permission(self, request, view, obj):
        return self.has_permission(request, view)


class IsHeadOfOperations(permissions.BasePermission):
    """Allows access only to Head of Operations. """
    message = "Only Head of Operations are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'Head of Operations')

    def has_object_permission(self, request, view, obj):
        return self.has_permission(request, view)


class IsAccounts(permissions.BasePermission):
    """Allows access only to Accounts. """
    message = "Only Accounts are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'Accounts')

    def has_object_permission(self, request, view, obj):
        return self.has_permission(request, view)


class IsReconciliation(permissions.BasePermission):
    """Allows access only to CSO. """
    message = "Only Reconciliation are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'Reconciliation')

    def has_object_permission(self, request, view, obj):
        return self.has_permission(request, view)


class IsMaComm(permissions.BasePermission):
    """Allows access only to MaComm. """
    message = "Only MaComm are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'MaComm')

    def has_object_permission(self, request, view, obj):
        return self.has_permission(request, view)


class IsCustomerSupportSupervisor(permissions.BasePermission):
    """Allows access only to Customer Support Supervisor. """
    message = "Only Customer Support Supervisor are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'Customer Support Supervisor')

    def has_object_permission(self, request, view, obj):
        return self.has_permission(request, view)


class IsCustomerSupportOfficer(permissions.BasePermission):
    """Allows access only to Customer Support Officer. """
    message = "Only Customer Support Officer are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'Customer Support Officer')

    def has_object_permission(self, request, view, obj):
        return self.has_permission(request, view)


class IsHeadInternalControl(permissions.BasePermission):
    """Allows access only to Head Internal Control. """
    message = "Only Head Internal Control are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'Head Internal Control')

    def has_object_permission(self, request, view, obj):
        return self.has_permission(request, view)


class IsInternalControlOfficer(permissions.BasePermission):
    """Allows access only to Internal Control Officer. """
    message = "Only Internal Control Officer are authorized to perform this action."

    def has_permission(self, request, view):
        return bool(request.user.is_authenticated and request.user.role == 'Internal Control Officer')

    def has_object_permission(self, request, view, obj):
        return self.has_permission(request, view)


class IsSafeIPAddress(permissions.BasePermission):
    """
    Ensure the request's IP address is on the safe list configured in Django settings.
    """

    @staticmethod
    def get_client_ip(request):
        ip_addresses = [request.META.get('REMOTE_ADDR', ''), request.META.get('HTTP_X_FORWARDED_FOR', '')]
        return [addr for addr in ip_addresses if addr]

    def has_permission(self, request, view):

        remote_addresses = self.get_client_ip(request)
        logger.info({'remote_addresses': remote_addresses})
        if not settings.DEBUG:
            return any(element in remote_addresses for element in settings.ATHENA_SAFE_LIST_IPS)
        return True


from rest_framework import exceptions


def clean_phone_number(phone_number: str) -> str:
    if len(phone_number) < 10 or len(phone_number) > 14:
        raise exceptions.ValidationError({'phone': f'Invalid phone number: {phone_number}'})
    if phone_number.startswith('+234') and len(phone_number) < 14:
        raise exceptions.ValidationError({'phone': f'Invalid phone number: {phone_number}'})
    if phone_number.startswith('234') and len(phone_number) == 13:
        return f"+{phone_number}"
    if phone_number.startswith('0') and len(phone_number) == 11:
        return f"+234{phone_number[1:]}"
    return phone_number


def clean_termii_phone(phone_number: str) -> str:
    return clean_phone_number(phone_number)[1:]



from dataclasses import dataclass
from functools import wraps

from rest_framework.exceptions import PermissionDenied


@dataclass
class PERMISSIONS:
    CAN_VIEW_AGENT_DASHBOARD = "can_view_agent_dashboard"

    # user viewset
    CAN_INVITE_USER = "can_invite_user"
    CAN_VIEW_USERS_LIST = "can_view_users_list"
    CAN_VIEW_USER = "can_view_user"
    CAN_UPDATE_USER = "can_update_user"
    CAN_DELETE_USER = "can_delete_user"
    CAN_RE_INVITE_USER = "can_re_invite_user"
    CAN_UPDATE_USER_STATUS = "can_update_user_status"
    CAN_VIEW_DASHBOARD = "can_view_dashboard"

    # teams
    CAN_INVITE_TEAM_USER = "can_invite_team_user"
    CAN_VIEW_TEAMS_LIST = "can_view_teams_list"
    CAN_VIEW_TEAM = "can_view_team"
    CAN_UPDATE_TEAM = "can_update_team"
    CAN_RETRIEVE_DIRECT_REPORT = "can_retrieve_direct_report"
    CAN_DISABLE_TEAM_MEMBER = "can_disable_team_member"
    CAN_RETRIEVE_FUNCTIONAL_DIRECT_REPORTS_STATS = "can_retrieve_functional_direct_reports_stats"
    CAN_REACTIVE_TEAM = "can_reactive_team"
    CAN_VIEW_TEAM_STATS = "can_view_team_stats"

    # agent
    CAN_BLOCK_AGENT = "can_block_agent"
    CAN_VIEW_AGENT_LIST = "can_view_agent_list"
    CAN_CREATE_AGENT = "can_create_agent"

    # wallet settings
    CAN_CREATE_TERMINAL_PROFILE = "can_create_terminal_profile"
    CAN_EDIT_TERMINAL_PROFILE = "can_edit_terminal_profile"
    CAN_VIEW_TERMINAL_PROFILE = "can_view_terminal_profile"
    CAN_VIEW_TERMINAL_PROFILE_LIST = "can_view_terminal_profile_list"
    CAN_DELETE_TERMINAL_PROFILE = "can_delete_terminal_profile"

    CAN_CREATE_VAS_FEE = "can_create_vas_fee"
    CAN_VIEW_VAS_FEE = "can_view_vas_fee"
    CAN_EDIT_VAS_FEE = "can_edit_vas_fee"

    CAN_MANAGE_SETTLEMENT_BANK = "can_manage_settlement_bank"
    CAN_VIEW_SETTLEMENT_BANK = "can_view_settlement_bank"
    CAN_MANAGE_NIBBS_BANK = "can_manage_nibbs_bank"
    CAN_VIEW_NIBBS_BANK = "can_view_nibbs_bank"
    CAN_MANAGE_BANKS = "can_manage_banks"
    CAN_VIEW_BANKS = "can_view_banks"

    CAN_MANAGE_WEMA_COLLECTION = "can_manage_wema_collection"
    CAN_VIEW_WEMA_COLLECTION = "can_view_wema_collection"

    CAN_MANAGE_WITHDRAWAL_ROUTING = "can_manage_withdrawal_routing"
    CAN_VIEW_WITHDRAWAL_ROUTING = "can_view_withdrawal_routing"
    CAN_MANAGE_VAS_SETTINGS = "can_manage_vas_settings"
    CAN_VIEW_VAS_SETTINGS = "can_view_vas_settings"
    CAN_MANAGE_TRANSFER_PROVIDER_SETTINGS = "can_manage_transfer_provider_settings"
    CAN_VIEW_TRANSFER_PROVIDER_SETTINGS = "can_view_transfer_provider_settings"

    # transactions
    CAN_VIEW_TRANSACTION = "can_view_transaction"
    CAN_REVERSE_TRANSACTION = "can_reverse_transaction"
    CAN_BULK_REVERSE_TRANSACTION = "can_bulk_reverse_transaction"
    CAN_INITIATE_TRANSFER = "can_initiate_transfer"
    CAN_INITIATE_VAS = "can_initiate_vas"
    CAN_INITIATE_CASHOUT = "can_initiate_cashout"
    CAN_INITIATE_PRODUCT = "can_initiate_product"
    CAN_INITIATE_COMMISSION_PAYOUT = "can_initiate_commission_payout"
    CAN_DEBIT_CREDIT_TERMINAL = "can_debit_credit_terminal"
    CAN_FINALISE_TRANSACTION = "can_finalise_transaction"
    CAN_GET_TRANSACTION_STATS = "can_get_transaction_stats"
    CAN_SEND_STATEMENT = "can_send_statement"
    CAN_DOWNLOAD_STATEMENT = "can_download_statement"
    CAN_VIEW_SUCCESS_RATES = "can_view_success_rates"
    CAN_PAY_FOR_TERMINAL = "can_pay_for_terminal"

    # wallet
    CAN_CREATE_WALLET = "can_create_wallet"
    CAN_VIEW_WALLET = "can_view_wallet"
    CAN_VIEW_WALLET_LIST = "can_view_wallet_list"
    CAN_UPDATE_WALLET = "can_update_wallet"
    CAN_CREATE_TRANSACTION_PIN = "can_create_transaction_pin"
    CAN_UPDATE_WALLET_KYC_LEVEL = "can_update_wallet_kyc_level"
    CAN_RETRIEVE_TERMINAL = "can_retrieve_terminal"
    CAN_REPLACE_TERMINAL = "can_replace_terminal"
    CAN_UPDATE_WALLET_TERMINAL_PROFILE = "can_update_wallet_terminal_profile"
    CAN_BULK_UPDATE_WALLET_TERMINAL_PROFILE = "can_bulk_update_wallet_terminal_profile"
    CAN_BLOCK_UNBLOCK_TERMINAL = "can_block_unblock_terminal"
    CAN_BLOCK_UNBLOCK_USER = "can_block_unblock_user"
    CAN_GET_WALLET_STATS = "can_get_wallet_stats"
    CAN_GET_COMMISSION_STATS = "can_get_commission_stats"

    # terminal
    CAN_GET_TERMINAL_STATS = "can_get_terminal_stats"
    CAN_GET_TERMINAL_DETAILS = "can_get_terminal_details"
    CAN_GET_TERMINAL_WALLET_TERMINAL_LIST = "can_get_terminal_wallet_terminal_list"


ROLE_PERMISSIONS = {

    "Super Admin": [
        PERMISSIONS.CAN_INVITE_USER,
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_INVITE_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_UPDATE_USER,
        PERMISSIONS.CAN_RE_INVITE_USER,
        PERMISSIONS.CAN_UPDATE_USER_STATUS,
        PERMISSIONS.CAN_VIEW_DASHBOARD,

        PERMISSIONS.CAN_INVITE_TEAM_USER,
        PERMISSIONS.CAN_VIEW_TEAMS_LIST,
        PERMISSIONS.CAN_VIEW_TEAM,
        PERMISSIONS.CAN_UPDATE_TEAM,
        PERMISSIONS.CAN_RETRIEVE_DIRECT_REPORT,
        PERMISSIONS.CAN_DISABLE_TEAM_MEMBER,
        PERMISSIONS.CAN_RETRIEVE_FUNCTIONAL_DIRECT_REPORTS_STATS,
        PERMISSIONS.CAN_REACTIVE_TEAM,
        PERMISSIONS.CAN_VIEW_TEAM_STATS,

        PERMISSIONS.CAN_BLOCK_AGENT,
        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_CREATE_AGENT,

        PERMISSIONS.CAN_CREATE_TERMINAL_PROFILE,
        PERMISSIONS.CAN_EDIT_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE_LIST,
        PERMISSIONS.CAN_DELETE_TERMINAL_PROFILE,

        PERMISSIONS.CAN_CREATE_VAS_FEE,
        PERMISSIONS.CAN_VIEW_VAS_FEE,
        PERMISSIONS.CAN_EDIT_VAS_FEE,

        PERMISSIONS.CAN_MANAGE_SETTLEMENT_BANK,
        PERMISSIONS.CAN_MANAGE_NIBBS_BANK,
        PERMISSIONS.CAN_MANAGE_BANKS,

        PERMISSIONS.CAN_MANAGE_WEMA_COLLECTION,

        PERMISSIONS.CAN_MANAGE_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_VIEW_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_MANAGE_VAS_SETTINGS,
        PERMISSIONS.CAN_VIEW_VAS_SETTINGS,
        PERMISSIONS.CAN_MANAGE_TRANSFER_PROVIDER_SETTINGS,
        PERMISSIONS.CAN_VIEW_TRANSFER_PROVIDER_SETTINGS,

        # transactions
        PERMISSIONS.CAN_VIEW_TRANSACTION,
        PERMISSIONS.CAN_REVERSE_TRANSACTION,
        PERMISSIONS.CAN_BULK_REVERSE_TRANSACTION,
        PERMISSIONS.CAN_INITIATE_TRANSFER,
        PERMISSIONS.CAN_INITIATE_VAS,
        PERMISSIONS.CAN_INITIATE_CASHOUT,
        PERMISSIONS.CAN_INITIATE_PRODUCT,
        PERMISSIONS.CAN_INITIATE_COMMISSION_PAYOUT,
        PERMISSIONS.CAN_DEBIT_CREDIT_TERMINAL,
        PERMISSIONS.CAN_FINALISE_TRANSACTION,
        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_SEND_STATEMENT,
        PERMISSIONS.CAN_DOWNLOAD_STATEMENT,
        PERMISSIONS.CAN_VIEW_SUCCESS_RATES,
        PERMISSIONS.CAN_PAY_FOR_TERMINAL,

        # wallet
        PERMISSIONS.CAN_CREATE_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET_LIST,
        PERMISSIONS.CAN_UPDATE_WALLET,
        PERMISSIONS.CAN_CREATE_TRANSACTION_PIN,
        PERMISSIONS.CAN_UPDATE_WALLET_KYC_LEVEL,
        PERMISSIONS.CAN_RETRIEVE_TERMINAL,
        PERMISSIONS.CAN_REPLACE_TERMINAL,
        PERMISSIONS.CAN_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BULK_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BLOCK_UNBLOCK_TERMINAL,
        PERMISSIONS.CAN_BLOCK_UNBLOCK_USER,
        PERMISSIONS.CAN_GET_WALLET_STATS,
        PERMISSIONS.CAN_GET_COMMISSION_STATS,

        # terminal
        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ],
    "Admin": [
        PERMISSIONS.CAN_INVITE_USER,
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_INVITE_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_UPDATE_USER,
        PERMISSIONS.CAN_RE_INVITE_USER,
        PERMISSIONS.CAN_UPDATE_USER_STATUS,
        PERMISSIONS.CAN_VIEW_DASHBOARD,

        PERMISSIONS.CAN_INVITE_TEAM_USER,
        PERMISSIONS.CAN_VIEW_TEAMS_LIST,
        PERMISSIONS.CAN_VIEW_TEAM,
        PERMISSIONS.CAN_UPDATE_TEAM,
        PERMISSIONS.CAN_RETRIEVE_DIRECT_REPORT,
        PERMISSIONS.CAN_DISABLE_TEAM_MEMBER,
        PERMISSIONS.CAN_RETRIEVE_FUNCTIONAL_DIRECT_REPORTS_STATS,
        PERMISSIONS.CAN_REACTIVE_TEAM,
        PERMISSIONS.CAN_VIEW_TEAM_STATS,

        PERMISSIONS.CAN_BLOCK_AGENT,
        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_CREATE_AGENT,

        PERMISSIONS.CAN_CREATE_TERMINAL_PROFILE,
        PERMISSIONS.CAN_EDIT_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE_LIST,
        PERMISSIONS.CAN_DELETE_TERMINAL_PROFILE,

        PERMISSIONS.CAN_CREATE_VAS_FEE,
        PERMISSIONS.CAN_VIEW_VAS_FEE,
        PERMISSIONS.CAN_EDIT_VAS_FEE,

        PERMISSIONS.CAN_MANAGE_SETTLEMENT_BANK,
        PERMISSIONS.CAN_MANAGE_NIBBS_BANK,
        PERMISSIONS.CAN_MANAGE_BANKS,

        PERMISSIONS.CAN_MANAGE_WEMA_COLLECTION,

        PERMISSIONS.CAN_MANAGE_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_VIEW_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_MANAGE_VAS_SETTINGS,
        PERMISSIONS.CAN_VIEW_VAS_SETTINGS,
        PERMISSIONS.CAN_MANAGE_TRANSFER_PROVIDER_SETTINGS,
        PERMISSIONS.CAN_VIEW_TRANSFER_PROVIDER_SETTINGS,

        # transactions
        PERMISSIONS.CAN_VIEW_TRANSACTION,
        PERMISSIONS.CAN_REVERSE_TRANSACTION,
        PERMISSIONS.CAN_BULK_REVERSE_TRANSACTION,
        PERMISSIONS.CAN_INITIATE_TRANSFER,
        PERMISSIONS.CAN_INITIATE_VAS,
        PERMISSIONS.CAN_INITIATE_CASHOUT,
        PERMISSIONS.CAN_INITIATE_PRODUCT,
        PERMISSIONS.CAN_INITIATE_COMMISSION_PAYOUT,
        PERMISSIONS.CAN_DEBIT_CREDIT_TERMINAL,
        PERMISSIONS.CAN_FINALISE_TRANSACTION,
        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_SEND_STATEMENT,
        PERMISSIONS.CAN_DOWNLOAD_STATEMENT,
        PERMISSIONS.CAN_VIEW_SUCCESS_RATES,
        PERMISSIONS.CAN_PAY_FOR_TERMINAL,

        # wallet
        PERMISSIONS.CAN_CREATE_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET_LIST,
        PERMISSIONS.CAN_UPDATE_WALLET,
        PERMISSIONS.CAN_CREATE_TRANSACTION_PIN,
        PERMISSIONS.CAN_UPDATE_WALLET_KYC_LEVEL,
        PERMISSIONS.CAN_RETRIEVE_TERMINAL,
        PERMISSIONS.CAN_REPLACE_TERMINAL,
        PERMISSIONS.CAN_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BULK_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BLOCK_UNBLOCK_TERMINAL,
        PERMISSIONS.CAN_BLOCK_UNBLOCK_USER,
        PERMISSIONS.CAN_GET_WALLET_STATS,
        PERMISSIONS.CAN_GET_COMMISSION_STATS,

        # terminal
        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,
    ],
    "Head of Operations": [
        PERMISSIONS.CAN_INVITE_USER,
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_INVITE_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_UPDATE_USER,
        PERMISSIONS.CAN_RE_INVITE_USER,
        PERMISSIONS.CAN_UPDATE_USER_STATUS,
        PERMISSIONS.CAN_VIEW_DASHBOARD,

        PERMISSIONS.CAN_INVITE_TEAM_USER,
        PERMISSIONS.CAN_VIEW_TEAMS_LIST,
        PERMISSIONS.CAN_VIEW_TEAM,
        PERMISSIONS.CAN_UPDATE_TEAM,
        PERMISSIONS.CAN_RETRIEVE_DIRECT_REPORT,
        PERMISSIONS.CAN_DISABLE_TEAM_MEMBER,
        PERMISSIONS.CAN_RETRIEVE_FUNCTIONAL_DIRECT_REPORTS_STATS,
        PERMISSIONS.CAN_REACTIVE_TEAM,
        PERMISSIONS.CAN_VIEW_TEAM_STATS,

        PERMISSIONS.CAN_BLOCK_AGENT,
        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_CREATE_AGENT,

        PERMISSIONS.CAN_CREATE_TERMINAL_PROFILE,
        PERMISSIONS.CAN_EDIT_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE_LIST,
        PERMISSIONS.CAN_DELETE_TERMINAL_PROFILE,

        PERMISSIONS.CAN_CREATE_VAS_FEE,
        PERMISSIONS.CAN_VIEW_VAS_FEE,
        PERMISSIONS.CAN_EDIT_VAS_FEE,

        PERMISSIONS.CAN_MANAGE_SETTLEMENT_BANK,
        PERMISSIONS.CAN_MANAGE_NIBBS_BANK,
        PERMISSIONS.CAN_MANAGE_BANKS,

        PERMISSIONS.CAN_MANAGE_WEMA_COLLECTION,

        PERMISSIONS.CAN_MANAGE_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_VIEW_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_MANAGE_VAS_SETTINGS,
        PERMISSIONS.CAN_VIEW_VAS_SETTINGS,
        PERMISSIONS.CAN_MANAGE_TRANSFER_PROVIDER_SETTINGS,
        PERMISSIONS.CAN_VIEW_TRANSFER_PROVIDER_SETTINGS,

        # transactions
        PERMISSIONS.CAN_VIEW_TRANSACTION,
        PERMISSIONS.CAN_REVERSE_TRANSACTION,
        PERMISSIONS.CAN_BULK_REVERSE_TRANSACTION,
        PERMISSIONS.CAN_INITIATE_TRANSFER,
        PERMISSIONS.CAN_INITIATE_VAS,
        PERMISSIONS.CAN_INITIATE_CASHOUT,
        PERMISSIONS.CAN_INITIATE_PRODUCT,
        PERMISSIONS.CAN_INITIATE_COMMISSION_PAYOUT,
        PERMISSIONS.CAN_DEBIT_CREDIT_TERMINAL,
        PERMISSIONS.CAN_FINALISE_TRANSACTION,
        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_SEND_STATEMENT,
        PERMISSIONS.CAN_DOWNLOAD_STATEMENT,
        PERMISSIONS.CAN_VIEW_SUCCESS_RATES,
        PERMISSIONS.CAN_PAY_FOR_TERMINAL,

        # wallet
        PERMISSIONS.CAN_CREATE_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET_LIST,
        PERMISSIONS.CAN_UPDATE_WALLET,
        PERMISSIONS.CAN_CREATE_TRANSACTION_PIN,
        PERMISSIONS.CAN_UPDATE_WALLET_KYC_LEVEL,
        PERMISSIONS.CAN_RETRIEVE_TERMINAL,
        PERMISSIONS.CAN_REPLACE_TERMINAL,
        PERMISSIONS.CAN_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BULK_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BLOCK_UNBLOCK_TERMINAL,
        PERMISSIONS.CAN_BLOCK_UNBLOCK_USER,
        PERMISSIONS.CAN_GET_WALLET_STATS,
        PERMISSIONS.CAN_GET_COMMISSION_STATS,

        # terminal
        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,
    ],
    "Admin View Only": [
        PERMISSIONS.CAN_INVITE_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_UPDATE_USER,
        PERMISSIONS.CAN_RE_INVITE_USER,
        PERMISSIONS.CAN_UPDATE_USER_STATUS,
        PERMISSIONS.CAN_VIEW_DASHBOARD,

        PERMISSIONS.CAN_INVITE_TEAM_USER,
        PERMISSIONS.CAN_VIEW_TEAMS_LIST,
        PERMISSIONS.CAN_VIEW_TEAM,
        PERMISSIONS.CAN_UPDATE_TEAM,
        PERMISSIONS.CAN_RETRIEVE_DIRECT_REPORT,
        PERMISSIONS.CAN_DISABLE_TEAM_MEMBER,
        PERMISSIONS.CAN_RETRIEVE_FUNCTIONAL_DIRECT_REPORTS_STATS,
        PERMISSIONS.CAN_REACTIVE_TEAM,
        PERMISSIONS.CAN_VIEW_TEAM_STATS,

        PERMISSIONS.CAN_BLOCK_AGENT,
        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_CREATE_AGENT,

        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE_LIST,

        PERMISSIONS.CAN_VIEW_VAS_FEE,

        PERMISSIONS.CAN_VIEW_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_VIEW_VAS_SETTINGS,
        PERMISSIONS.CAN_VIEW_TRANSFER_PROVIDER_SETTINGS,

        PERMISSIONS.CAN_MANAGE_WEMA_COLLECTION,

        PERMISSIONS.CAN_MANAGE_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_MANAGE_VAS_SETTINGS,
        PERMISSIONS.CAN_MANAGE_TRANSFER_PROVIDER_SETTINGS,

        # transactions
        PERMISSIONS.CAN_VIEW_TRANSACTION,
        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_VIEW_SUCCESS_RATES,

        # wallet
        PERMISSIONS.CAN_VIEW_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET_LIST,
        PERMISSIONS.CAN_GET_WALLET_STATS,
        PERMISSIONS.CAN_GET_COMMISSION_STATS,

        # terminal
        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ],
    "Divisional Head": [
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_INVITE_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_RE_INVITE_USER,
        PERMISSIONS.CAN_VIEW_DASHBOARD,

        PERMISSIONS.CAN_VIEW_TEAMS_LIST,
        PERMISSIONS.CAN_VIEW_TEAM,
        PERMISSIONS.CAN_RETRIEVE_DIRECT_REPORT,
        PERMISSIONS.CAN_RETRIEVE_FUNCTIONAL_DIRECT_REPORTS_STATS,
        PERMISSIONS.CAN_VIEW_TEAM_STATS,

        PERMISSIONS.CAN_BLOCK_AGENT,
        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_CREATE_AGENT,

        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE_LIST,

        PERMISSIONS.CAN_VIEW_VAS_FEE,

        PERMISSIONS.CAN_VIEW_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_VIEW_VAS_SETTINGS,
        PERMISSIONS.CAN_VIEW_TRANSFER_PROVIDER_SETTINGS,

        # transactions
        PERMISSIONS.CAN_VIEW_TRANSACTION,
        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_SEND_STATEMENT,
        PERMISSIONS.CAN_DOWNLOAD_STATEMENT,
        PERMISSIONS.CAN_VIEW_SUCCESS_RATES,
        PERMISSIONS.CAN_PAY_FOR_TERMINAL,

        # wallet
        PERMISSIONS.CAN_CREATE_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET_LIST,
        PERMISSIONS.CAN_UPDATE_WALLET,
        PERMISSIONS.CAN_UPDATE_WALLET_KYC_LEVEL,
        PERMISSIONS.CAN_RETRIEVE_TERMINAL,
        PERMISSIONS.CAN_REPLACE_TERMINAL,
        PERMISSIONS.CAN_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BULK_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BLOCK_UNBLOCK_TERMINAL,
        PERMISSIONS.CAN_BLOCK_UNBLOCK_USER,
        PERMISSIONS.CAN_GET_WALLET_STATS,
        PERMISSIONS.CAN_GET_COMMISSION_STATS,

        # terminal
        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,
    ],
    "Agent Network Manager": [
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_VIEW_DASHBOARD,

        PERMISSIONS.CAN_BLOCK_AGENT,
        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_CREATE_AGENT,

        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE_LIST,

        PERMISSIONS.CAN_VIEW_VAS_FEE,

        PERMISSIONS.CAN_VIEW_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_VIEW_VAS_SETTINGS,
        PERMISSIONS.CAN_VIEW_TRANSFER_PROVIDER_SETTINGS,

        # transactions
        PERMISSIONS.CAN_VIEW_TRANSACTION,
        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_SEND_STATEMENT,
        PERMISSIONS.CAN_DOWNLOAD_STATEMENT,
        PERMISSIONS.CAN_VIEW_SUCCESS_RATES,
        PERMISSIONS.CAN_PAY_FOR_TERMINAL,

        # wallet
        PERMISSIONS.CAN_CREATE_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET_LIST,
        PERMISSIONS.CAN_UPDATE_WALLET,
        PERMISSIONS.CAN_UPDATE_WALLET_KYC_LEVEL,
        PERMISSIONS.CAN_RETRIEVE_TERMINAL,
        PERMISSIONS.CAN_REPLACE_TERMINAL,
        PERMISSIONS.CAN_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BULK_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BLOCK_UNBLOCK_TERMINAL,
        PERMISSIONS.CAN_BLOCK_UNBLOCK_USER,
        PERMISSIONS.CAN_GET_WALLET_STATS,
        PERMISSIONS.CAN_GET_COMMISSION_STATS,

        # terminal
        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ],
    "Regional Head": [
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_INVITE_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_RE_INVITE_USER,
        PERMISSIONS.CAN_VIEW_DASHBOARD,

        PERMISSIONS.CAN_VIEW_TEAMS_LIST,
        PERMISSIONS.CAN_VIEW_TEAM,
        PERMISSIONS.CAN_RETRIEVE_DIRECT_REPORT,
        PERMISSIONS.CAN_RETRIEVE_FUNCTIONAL_DIRECT_REPORTS_STATS,
        PERMISSIONS.CAN_VIEW_TEAM_STATS,

        PERMISSIONS.CAN_BLOCK_AGENT,
        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_CREATE_AGENT,

        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE_LIST,

        PERMISSIONS.CAN_VIEW_VAS_FEE,

        PERMISSIONS.CAN_VIEW_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_VIEW_VAS_SETTINGS,
        PERMISSIONS.CAN_VIEW_TRANSFER_PROVIDER_SETTINGS,

        # transactions
        PERMISSIONS.CAN_VIEW_TRANSACTION,
        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_SEND_STATEMENT,
        PERMISSIONS.CAN_DOWNLOAD_STATEMENT,
        PERMISSIONS.CAN_VIEW_SUCCESS_RATES,
        PERMISSIONS.CAN_PAY_FOR_TERMINAL,

        # wallet
        PERMISSIONS.CAN_CREATE_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET_LIST,
        PERMISSIONS.CAN_UPDATE_WALLET,
        PERMISSIONS.CAN_UPDATE_WALLET_KYC_LEVEL,
        PERMISSIONS.CAN_RETRIEVE_TERMINAL,
        PERMISSIONS.CAN_REPLACE_TERMINAL,
        PERMISSIONS.CAN_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BULK_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BLOCK_UNBLOCK_TERMINAL,
        PERMISSIONS.CAN_BLOCK_UNBLOCK_USER,
        PERMISSIONS.CAN_GET_WALLET_STATS,
        PERMISSIONS.CAN_GET_COMMISSION_STATS,

        # terminal
        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ],
    "Relationship Officer": [
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_VIEW_DASHBOARD,

        PERMISSIONS.CAN_VIEW_TEAMS_LIST,
        PERMISSIONS.CAN_VIEW_TEAM,
        PERMISSIONS.CAN_RETRIEVE_DIRECT_REPORT,
        PERMISSIONS.CAN_RETRIEVE_FUNCTIONAL_DIRECT_REPORTS_STATS,
        PERMISSIONS.CAN_VIEW_TEAM_STATS,

        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_CREATE_AGENT,

        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE_LIST,

        PERMISSIONS.CAN_VIEW_VAS_FEE,

        PERMISSIONS.CAN_VIEW_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_VIEW_VAS_SETTINGS,
        PERMISSIONS.CAN_VIEW_TRANSFER_PROVIDER_SETTINGS,

        # transactions
        PERMISSIONS.CAN_VIEW_TRANSACTION,
        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_SEND_STATEMENT,
        PERMISSIONS.CAN_DOWNLOAD_STATEMENT,
        PERMISSIONS.CAN_VIEW_SUCCESS_RATES,
        PERMISSIONS.CAN_PAY_FOR_TERMINAL,

        # wallet
        PERMISSIONS.CAN_CREATE_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET_LIST,
        PERMISSIONS.CAN_UPDATE_WALLET,
        PERMISSIONS.CAN_UPDATE_WALLET_KYC_LEVEL,
        PERMISSIONS.CAN_RETRIEVE_TERMINAL,
        PERMISSIONS.CAN_REPLACE_TERMINAL,
        PERMISSIONS.CAN_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BULK_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BLOCK_UNBLOCK_TERMINAL,
        PERMISSIONS.CAN_BLOCK_UNBLOCK_USER,
        PERMISSIONS.CAN_GET_WALLET_STATS,
        PERMISSIONS.CAN_GET_COMMISSION_STATS,

        # terminal
        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ],
    "Merchant": [
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_UPDATE_USER,
        PERMISSIONS.CAN_VIEW_DASHBOARD,

        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_CREATE_AGENT,

        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE_LIST,

        PERMISSIONS.CAN_VIEW_VAS_FEE,

        PERMISSIONS.CAN_VIEW_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_VIEW_VAS_SETTINGS,
        PERMISSIONS.CAN_VIEW_TRANSFER_PROVIDER_SETTINGS,

        # transactions
        PERMISSIONS.CAN_VIEW_TRANSACTION,
        PERMISSIONS.CAN_INITIATE_TRANSFER,
        PERMISSIONS.CAN_INITIATE_VAS,
        PERMISSIONS.CAN_INITIATE_CASHOUT,
        PERMISSIONS.CAN_INITIATE_PRODUCT,
        PERMISSIONS.CAN_INITIATE_COMMISSION_PAYOUT,
        PERMISSIONS.CAN_FINALISE_TRANSACTION,
        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_SEND_STATEMENT,
        PERMISSIONS.CAN_DOWNLOAD_STATEMENT,
        PERMISSIONS.CAN_VIEW_SUCCESS_RATES,
        PERMISSIONS.CAN_PAY_FOR_TERMINAL,

        # wallet
        PERMISSIONS.CAN_CREATE_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET_LIST,
        PERMISSIONS.CAN_UPDATE_WALLET,
        PERMISSIONS.CAN_CREATE_TRANSACTION_PIN,
        PERMISSIONS.CAN_UPDATE_WALLET_KYC_LEVEL,
        PERMISSIONS.CAN_RETRIEVE_TERMINAL,
        PERMISSIONS.CAN_REPLACE_TERMINAL,
        PERMISSIONS.CAN_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BULK_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_GET_WALLET_STATS,
        PERMISSIONS.CAN_GET_COMMISSION_STATS,

        # terminal
        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ],
    "Super Agent": [
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_UPDATE_USER,
        PERMISSIONS.CAN_VIEW_DASHBOARD,

        PERMISSIONS.CAN_BLOCK_AGENT,
        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_CREATE_AGENT,

        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE_LIST,

        PERMISSIONS.CAN_VIEW_VAS_FEE,

        PERMISSIONS.CAN_VIEW_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_VIEW_VAS_SETTINGS,
        PERMISSIONS.CAN_VIEW_TRANSFER_PROVIDER_SETTINGS,

        # transactions
        PERMISSIONS.CAN_VIEW_TRANSACTION,
        PERMISSIONS.CAN_INITIATE_TRANSFER,
        PERMISSIONS.CAN_INITIATE_VAS,
        PERMISSIONS.CAN_INITIATE_CASHOUT,
        PERMISSIONS.CAN_INITIATE_PRODUCT,
        PERMISSIONS.CAN_INITIATE_COMMISSION_PAYOUT,
        PERMISSIONS.CAN_FINALISE_TRANSACTION,
        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_SEND_STATEMENT,
        PERMISSIONS.CAN_DOWNLOAD_STATEMENT,
        PERMISSIONS.CAN_VIEW_SUCCESS_RATES,
        PERMISSIONS.CAN_PAY_FOR_TERMINAL,

        # wallet
        PERMISSIONS.CAN_CREATE_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET_LIST,
        PERMISSIONS.CAN_UPDATE_WALLET,
        PERMISSIONS.CAN_CREATE_TRANSACTION_PIN,
        PERMISSIONS.CAN_UPDATE_WALLET_KYC_LEVEL,
        PERMISSIONS.CAN_RETRIEVE_TERMINAL,
        PERMISSIONS.CAN_REPLACE_TERMINAL,
        PERMISSIONS.CAN_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BULK_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_GET_WALLET_STATS,
        PERMISSIONS.CAN_GET_COMMISSION_STATS,

        # terminal
        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ],
    "Agent": [
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_VIEW_USERS_LIST,
        PERMISSIONS.CAN_UPDATE_USER,
        PERMISSIONS.CAN_VIEW_DASHBOARD,

        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_CREATE_AGENT,

        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE_LIST,

        PERMISSIONS.CAN_VIEW_VAS_FEE,

        PERMISSIONS.CAN_VIEW_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_VIEW_VAS_SETTINGS,
        PERMISSIONS.CAN_VIEW_TRANSFER_PROVIDER_SETTINGS,

        # transactions
        PERMISSIONS.CAN_VIEW_TRANSACTION,
        PERMISSIONS.CAN_INITIATE_TRANSFER,
        PERMISSIONS.CAN_INITIATE_VAS,
        PERMISSIONS.CAN_INITIATE_CASHOUT,
        PERMISSIONS.CAN_INITIATE_PRODUCT,
        PERMISSIONS.CAN_INITIATE_COMMISSION_PAYOUT,
        PERMISSIONS.CAN_FINALISE_TRANSACTION,
        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_SEND_STATEMENT,
        PERMISSIONS.CAN_DOWNLOAD_STATEMENT,
        PERMISSIONS.CAN_VIEW_SUCCESS_RATES,
        PERMISSIONS.CAN_PAY_FOR_TERMINAL,

        # wallet
        PERMISSIONS.CAN_CREATE_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET,
        PERMISSIONS.CAN_VIEW_WALLET_LIST,
        PERMISSIONS.CAN_UPDATE_WALLET,
        PERMISSIONS.CAN_CREATE_TRANSACTION_PIN,
        PERMISSIONS.CAN_UPDATE_WALLET_KYC_LEVEL,
        PERMISSIONS.CAN_RETRIEVE_TERMINAL,
        PERMISSIONS.CAN_REPLACE_TERMINAL,
        PERMISSIONS.CAN_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_BULK_UPDATE_WALLET_TERMINAL_PROFILE,
        PERMISSIONS.CAN_GET_WALLET_STATS,
        PERMISSIONS.CAN_GET_COMMISSION_STATS,

        # terminal
        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ],
    "Accounts": [
        PERMISSIONS.CAN_VIEW_DASHBOARD,
        PERMISSIONS.CAN_BLOCK_AGENT,
        PERMISSIONS.CAN_VIEW_USER,

        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_VIEW_TEAMS_LIST,
        PERMISSIONS.CAN_VIEW_TEAM_STATS,

        PERMISSIONS.CAN_VIEW_TRANSACTION,

        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ],
    "Reconciliation": [
        PERMISSIONS.CAN_VIEW_DASHBOARD,
        PERMISSIONS.CAN_BLOCK_AGENT,
        PERMISSIONS.CAN_VIEW_USER,

        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_VIEW_TEAMS_LIST,
        PERMISSIONS.CAN_VIEW_TEAM_STATS,

        PERMISSIONS.CAN_VIEW_TRANSACTION,

        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ],
    "MaComm": [
        PERMISSIONS.CAN_VIEW_DASHBOARD,
        PERMISSIONS.CAN_BLOCK_AGENT,
        PERMISSIONS.CAN_VIEW_USER,

        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_VIEW_TEAMS_LIST,
        PERMISSIONS.CAN_VIEW_TEAM_STATS,

        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ],
    "Customer Support Supervisor": [
        PERMISSIONS.CAN_VIEW_DASHBOARD,
        PERMISSIONS.CAN_BLOCK_AGENT,

        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_VIEW_TEAM,
        PERMISSIONS.CAN_CREATE_AGENT,
        PERMISSIONS.CAN_VIEW_TEAMS_LIST,
        PERMISSIONS.CAN_VIEW_TEAM_STATS,

        PERMISSIONS.CAN_CREATE_TERMINAL_PROFILE,
        PERMISSIONS.CAN_EDIT_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE,
        PERMISSIONS.CAN_VIEW_TERMINAL_PROFILE_LIST,

        PERMISSIONS.CAN_CREATE_VAS_FEE,
        PERMISSIONS.CAN_VIEW_VAS_FEE,

        PERMISSIONS.CAN_MANAGE_WEMA_COLLECTION,

        PERMISSIONS.CAN_VIEW_WITHDRAWAL_ROUTING,
        PERMISSIONS.CAN_VIEW_VAS_SETTINGS,
        PERMISSIONS.CAN_VIEW_TRANSFER_PROVIDER_SETTINGS,

        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_VIEW_TRANSACTION,

        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ],
    "Customer Support Officer": [
        PERMISSIONS.CAN_VIEW_DASHBOARD,
        PERMISSIONS.CAN_BLOCK_AGENT,
        PERMISSIONS.CAN_VIEW_TEAM,
        PERMISSIONS.CAN_CREATE_AGENT,
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_VIEW_TEAMS_LIST,
        PERMISSIONS.CAN_VIEW_TEAM_STATS,

        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_VIEW_TRANSACTION,

        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ],
    "Head Internal Control": [
        PERMISSIONS.CAN_VIEW_DASHBOARD,
        PERMISSIONS.CAN_VIEW_TEAMS_LIST,
        PERMISSIONS.CAN_VIEW_TEAM,
        PERMISSIONS.CAN_INVITE_TEAM_USER,
        PERMISSIONS.CAN_VIEW_TEAM_STATS,
        PERMISSIONS.CAN_RETRIEVE_DIRECT_REPORT,
        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_VIEW_USER,

        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_VIEW_TRANSACTION,

        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ],
    "Internal Control Officer": [
        PERMISSIONS.CAN_VIEW_DASHBOARD,
        PERMISSIONS.CAN_VIEW_AGENT_LIST,
        PERMISSIONS.CAN_VIEW_USER,
        PERMISSIONS.CAN_VIEW_TEAMS_LIST,
        PERMISSIONS.CAN_VIEW_TEAM_STATS,

        PERMISSIONS.CAN_GET_TRANSACTION_STATS,
        PERMISSIONS.CAN_VIEW_TRANSACTION,

        PERMISSIONS.CAN_GET_TERMINAL_STATS,
        PERMISSIONS.CAN_GET_TERMINAL_DETAILS,
        PERMISSIONS.CAN_GET_TERMINAL_WALLET_TERMINAL_LIST,

    ]
}


def role_has_permission(_role, _permission):
    role_permissions = ROLE_PERMISSIONS.get(_role, [])
    return _permission in role_permissions


def permission_required(perm, raise_exception=True):
    def decorator(view_func):
        @wraps(view_func)
        def wrapper(cls, request, *args, **kwargs):
            if isinstance(perm, str):
                perms = (perm,)
            else:
                perms = perm

            user = request.user

            def check_perm(user_obj, perm_list):
                return all(role_has_permission(user_obj.role, _perm) for _perm in perm_list)

            if user and check_perm(user, perms) is False and raise_exception:
                raise PermissionDenied
            return view_func(cls, request, *args, **kwargs)

        return wrapper

    return decorator



from django.conf import settings

from .identity.factory import IdentityFactory


def get_identity_data(identity_type, identity_number, image=None, user_id=None):
    factory = IdentityFactory()
    if hasattr(settings, 'PYKOLO_DEFAULT_IDENTITY_SERVICE'):
        identity_service = settings.PYKOLO_DEFAULT_IDENTITY_SERVICE
    else:
        identity_service = 'MOCK'
    service = factory.get_service(identity_service)
    match identity_type:
        case 'BVN':
            identity_response = service.lookup_bvn(identity_number)
        case 'NIN':
            identity_response = service.lookup_nin(identity_number)
        case 'BVN_SELFIE':
            identity_response = service.lookup_bvn_with_image(identity_number, image)
        case 'ADVANCED_BVN':
            identity_response = service.lookup_advanced_bvn(identity_number)
        case _:
            identity_response = None

    if identity_response:
        identity_response_data = identity_response.to_dict()
        return identity_response_data

    return None



from datetime import datetime, time

import django_filters


class BaseFilterSet(django_filters.FilterSet):

    def __init__(self, data=None, *args, **kwargs):
        # if filterset is bound, use initial values as defaults
        if data is not None:
            # get a mutable copy of the QueryDict
            data = data.copy()

            for name, f in self.base_filters.items():
                initial = f.extra.get('initial')

                # filter param is either missing or empty, use initial as default
                if not data.get(name) and initial:
                    data[name] = initial

        super().__init__(data, *args, **kwargs)


class DateFilter(django_filters.FilterSet):
    start = django_filters.DateFilter(field_name="created_at", lookup_expr="gte")
    end = django_filters.DateFilter(field_name="created_at", method='filter_end')

    def __init__(self, start_field="created_at", end_field="created_at", **kwargs):
        super().__init__(**kwargs)
        self.start = django_filters.DateFilter(field_name=start_field, lookup_expr="gte")
        self.end = django_filters.DateFilter(field_name=end_field, method='filter_end')

    def filter_end(self, queryset, name, value):
        end_date = datetime.combine(value, time.max)
        f = {f"{name}__lte": end_date}
        return queryset.filter(**f)



from rest_framework.exceptions import APIException


class ServiceUnavailable(APIException):
    status_code = 503
    default_detail = 'Service temporarily unavailable, try again later.'
    default_code = 'service_unavailable'


class UnprocessableException(APIException):
    status_code = 422
    default_detail = 'Request cannot be process, missing important configurations'
    default_code = 'unprocessable'


class NotFoundException(APIException):
    status_code = 404
    default_detail = 'Requested entity does not exist'
    default_code = 'not_found'


class PermissionDeniedException(APIException):
    status_code = 403
    default_detail = "You don't have the permission to access this resource "
    default_code = 'permission_denied'


class BadRequestException(APIException):
    status_code = 400
    default_detail = 'Bad Request '
    default_code = 'bad_request'


import requests
from django.conf import settings

from .base import BaseIdentityService, LookupResponse


class DojahIdentityService(BaseIdentityService):
    def __init__(self, **kwargs):
        self.base_url = settings.DOJAH_API_URL
        if settings.DOJAH_APP_ID is None or settings.DOJAH_API_KEY is None:
            raise Exception("Dojah API credentials not set")
        self.headers = {
            "AppId": settings.DOJAH_APP_ID,
            "Authorization": settings.DOJAH_API_KEY,
        }

    def lookup_advanced_bvn(self, bvn):
        params = {"bvn": bvn}
        response = requests.get(f"{self.base_url}/kyc/bvn/advance", headers=self.headers, params=params)
        if response.status_code == 200:
            data = response.json()
            data = data.get("entity")
            selfie_verification = data.get("selfie_verification", {})
            return LookupResponse(
                first_name=data.get("first_name"),
                last_name=data.get("last_name"),
                middle_name=data.get("middle_name"),
                date_of_birth=data.get("date_of_birth"),
                photo=data.get("image"),
                gender=data.get("gender"),  # Todo: parse to Male or Female
                phone_number=data.get("phone_number1"),
                match=selfie_verification.get('match', False),
                confidence_value=selfie_verification.get("confidence_value"),
                nin=data.get("nin"),
            )
        return None

    def lookup_bvn(self, bvn):
        params = {"bvn": bvn}
        response = requests.get(f"{self.base_url}/kyc/bvn/full", headers=self.headers, params=params)
        if response.status_code == 200:
            data = response.json()
            data = data.get("entity")
            selfie_verification = data.get("selfie_verification", {})
            return LookupResponse(
                first_name=data.get("first_name"),
                last_name=data.get("last_name"),
                middle_name=data.get("middle_name"),
                date_of_birth=data.get("date_of_birth"),
                photo=data.get("image"),
                gender=data.get("gender"),  # Todo: parse to Male or Female
                phone_number=data.get("phone_number1"),
                match=selfie_verification.get('match', False),
                confidence_value=selfie_verification.get("confidence_value")
            )
        return None

    def lookup_bvn_with_image(self, bvn, image):
        body = {"bvn": bvn, "selfie_image": image}
        url = f"{self.base_url}/kyc/bvn/verify"
        response = requests.post(url="https://api.dojah.io/api/v1/kyc/bvn/verify", headers=self.headers, json=body)
        if response.status_code == 200:
            data = response.json()
            data = data.get("entity")
            selfie_verification = data.get("selfie_verification", {})
            return LookupResponse(
                first_name=data.get("first_name"),
                last_name=data.get("last_name"),
                middle_name=data.get("middle_name"),
                date_of_birth=data.get("date_of_birth"),
                photo=data.get("image"),
                gender=data.get("gender"),  # Todo: parse to Male or Female
                phone_number=data.get("phone_number1"),
                match=selfie_verification.get('match', False),
                confidence_value=selfie_verification.get("confidence_value"),
            )
        return None

    def lookup_nin(self, nin):
        params = {"nin": nin}
        response = requests.get(f"{self.base_url}/kyc/nin", headers=self.headers, params=params)
        if response.status_code == 200:
            data = response.json()
            data = data.get("entity")
            selfie_verification = data.get("selfie_verification", {})
            return LookupResponse(
                first_name=data.get("first_name"),
                last_name=data.get("last_name"),
                middle_name=data.get("middle_name"),
                date_of_birth=data.get("date_of_birth"),
                photo=data.get("photo"),
                gender=data.get("gender"),  # Todo: parse to Male or Female
                phone_number=data.get("phone_number1"),
                match=selfie_verification.get('match', False),
                confidence_value=selfie_verification.get("confidence_value"),
            )
        return None



import datetime
import json
import logging

from django.urls import Resolver404, resolve

# Get the logger instance
logger = logging.getLogger(__name__)

class RequestResponseLoggerMiddleware:
    """
    Middleware to log requests and responses to the console.
    """

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        # Log request details
        start_time = datetime.datetime.now()

        # Process the request
        response = self.get_response(request)

        # Log response details
        duration = datetime.datetime.now() - start_time
        try:
            resolver_match = resolve(request.path_info)
            url_name = resolver_match.url_name
            namespace = resolver_match.namespace
        except Resolver404:
            resolver_match = None
            url_name = None
            namespace = None

        request_data = {}
        if request.content_type == "application/json":
            try:
                request_data = json.loads(request.body)
            except json.JSONDecodeError:
                request_data = request.body.decode('utf-8')
        elif request.content_type in ["multipart/form-data", "application/x-www-form-urlencoded"]:
            request_data = request.POST.dict()
            if request.FILES:
                request_data['files'] = {key: file.name for key, file in request.FILES.items()}

        if namespace != 'admin' and request.method in ['POST', 'PUT', 'PATCH', 'DELETE']:
            logger.info({
                'request': {'path': request.path, 'method': request.method, },
                'body': request_data,
                'status_code': response.status_code,
                'duration': duration.total_seconds(),
                'response': json.loads(response.content.decode('utf-8')) if response.content else None
            })
        return response
